searchData={"items":[{"type":"behaviour","title":"Finitomata","doc":"","ref":"Finitomata.html"},{"type":"behaviour","title":"Bird View - Finitomata","doc":"`Finitomata` provides a boilerplate for [FSM](https://en.wikipedia.org/wiki/Finite-state_machine) implementation, allowing to concentrate on the business logic rather than on the process management and transitions/events consistency tweaking.\n\nIt reads a description of the FSM from a string in [PlantUML](https://plantuml.com/en/state-diagram), [Mermaid](https://mermaid.live), or even custom format. \n\n> #","ref":"Finitomata.html#module-bird-view"},{"type":"behaviour","title":"Syntax Definition {: .tip} - Finitomata","doc":">\n> `Mermaid` **state diagram** format is literally the same as `PlantUML`, so if you want to use it, specify `syntax: :state_diagram` and\n> if you want to use **mermaid graph**, specify `syntax: :flowchart`. The latter is the default.\n\nBasically, it looks more or less like this\n\n### `PlantUML` / `:state_diagram`\n\n    [*] --> s1 : to_s1\n    s1 --> s2 : to_s2\n    s1 --> s3 : to_s3\n    s2 --> [*] : ok\n    s3 --> [*] : ok\n\n### `Mermaid` / `:flowchart`\n\n    s1 --> |to_s2| s2\n    s1 --> |to_s3| s3\n\n> #","ref":"Finitomata.html#module-syntax-definition-tip"},{"type":"behaviour","title":"Using `syntax: :flowchart` {: .tip} - Finitomata","doc":">\n> `Mermaid` does not allow to explicitly specify transitions (and hence event names)\n> from the starting state and to the end state(s), these states names are implicitly set to `:*`\n> and events to `:__start__` and `:__end__` respectively.\n\n`Finitomata` validates the FSM is consistent, namely it has a single initial state, one or more final states, and no orphan states. If everything is OK, it generates a `GenServer` that could be used both alone, and with provided supervision tree. This `GenServer` requires to implement six callbacks\n\n- `on_transition/4` — **mandatory**\n- `on_failure/3` — optional\n- `on_enter/2` — optional\n- `on_exit/2` — optional\n- `on_terminate/1` — optional\n- `on_timer/2` — optional\n\nAll the callbacks do have a default implementation, that would perfectly handle transitions having a single _to_ state and not requiring any additional business logic attached.\n\nUpon start, it moves to the next to initial state and sits there awaiting for the _transition request_. Then it would call an `on_transition/4` callback and move to the next state, or remain in the current one, according to the response.\n\nUpon reaching a final state, it would terminate itself. The process keeps all the history of states it went through, and might have a payload in its state.","ref":"Finitomata.html#module-using-syntax-flowchart-tip"},{"type":"behaviour","title":"Special Events - Finitomata","doc":"If the event name is ended with a bang (e. g. `idle --> |start!| started`) _and_\nthis event is the only one allowed from this state (there might be several transitions though,)\nit’d be considered as _determined_ and FSM will be transitioned into the new state instantly.\n\nIf the event name is ended with a question mark (e. g. `idle --> |start?| started`,)\nthe transition is considered as expected to fail; no `on_failure/2` callback would\nbe called on failure and no log warning will be printed.","ref":"Finitomata.html#module-special-events"},{"type":"behaviour","title":"FSM Tuning and Configuration - Finitomata","doc":"#","ref":"Finitomata.html#module-fsm-tuning-and-configuration"},{"type":"behaviour","title":"Recurrent Callback - Finitomata","doc":"If `timer: non_neg_integer()` option is passed to `use Finitomata`,\nthen `c:Finitomata.on_timer/2` callback will be executed recurrently.\nThis might be helpful if _FSM_ needs to update its state from the outside\nworld on regular basis.","ref":"Finitomata.html#module-recurrent-callback"},{"type":"behaviour","title":"Automatic FSM Termination - Finitomata","doc":"If `auto_terminate: true() | state() | [state()]` option is passed to `use Finitomata`,\nthe special `__end__` event to transition to the end state will be called automatically\nunder the hood, if the current state is either listed explicitly, or if the value of\nthe parameter is `true`.\n\n#","ref":"Finitomata.html#module-automatic-fsm-termination"},{"type":"behaviour","title":"Ensuring State Entry - Finitomata","doc":"If `ensure_entry: true() | [state()]` option is passed to `use Finitomata`, the transition\nattempt will be retried with `{:continue, {:transition, {event(), event_payload()}}}` message\nuntil succeeded. Neither `on_failure/2` callback is called nor warning message is logged.\n\nThe payload would be updated to hold `__retries__: pos_integer()` key. If the payload was not a map,\nit will be converted to a map `%{payload: payload}`.","ref":"Finitomata.html#module-ensuring-state-entry"},{"type":"behaviour","title":"Example - Finitomata","doc":"Let’s define the FSM instance\n\n```elixir\ndefmodule MyFSM do\n  @fsm \"\"\"\n  s1 --> |to_s2| s2\n  s1 --> |to_s3| s3\n  \"\"\"\n  use Finitomata, fsm: @fsm, syntax: :flowchart","ref":"Finitomata.html#module-example"},{"type":"behaviour","title":"or uncomment lines below for `:state_diagram` syntax - Finitomata","doc":"# @fsm \"\"\"\n  # [*] --> s1 : to_s1\n  # s1 --> s2 : to_s2\n  # s1 --> s3 : to_s3\n  # s2 --> [*] : __end__\n  # s3 --> [*] : __end__\n  # \"\"\"\n  # use Finitomata, fsm: @fsm, syntax: :state_diagram\n\n  @impl Finitomata\n  def on_transition(:s1, :to_s2, _event_payload, state_payload),\n    do: {:ok, :s2, state_payload}\nend\n```\n\nNow we can play with it a bit.\n\n```elixir\n# or embed into supervision tree using `Finitomata.child_spec()`\n{:ok, _pid} = Finitomata.start_link()\n\nFinitomata.start_fsm MyFSM, \"My first FSM\", %{foo: :bar}\nFinitomata.transition \"My first FSM\", {:to_s2, nil}\nFinitomata.state \"My first FSM\"                    \n#⇒ %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}}\n\nFinitomata.allowed? \"My first FSM\", :* # state\n#⇒ true\nFinitomata.responds? \"My first FSM\", :to_s2 # event\n#⇒ false\n\nFinitomata.transition \"My first FSM\", {:__end__, nil} # to final state\n#⇒ [info]  [◉ ⇄] [state: %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}}]\n\nFinitomata.alive? \"My first FSM\"\n#⇒ false\n```\n\nTypically, one would implement all the `on_transition/4` handlers, pattern matching on the state/event.\n\n\n\n> ### `use Finitomata` {: .info}\n>\n> When you `use Finitomata`, the Finitomata module will\n> do the following things for your module:\n>\n> - set `@behaviour Finitomata`\n> - compile and validate _FSM_ declaration, passed as `fsm:` keyword argument\n> - turn the module into `GenServer`\n> - inject default implementations of optional callbacks specified with\n>   `impl_for:` keyword argument (default: `:all`)\n> - expose a bunch of functions to query _FSM_ which would be visible in docs\n> - leaves `on_transition/4` mandatory callback to be implemeneted by\n>   the calling module and injects `before_compile` callback to validate\n>   the implementation (this option required `:finitomata` to be included\n>   in the list of compilers in `mix.exs`)","ref":"Finitomata.html#module-or-uncomment-lines-below-for-state_diagram-syntax"},{"type":"behaviour","title":"Options to `use Finitomata` - Finitomata","doc":"* `:fsm` (`t:String.t/0`) - Required. The FSM declaration with the syntax defined by `syntax` option.\n\n* `:syntax` - The FSM dialect parser to convert the declaration to internal FSM representation. The default value is `:flowchart`.\n\n* `:impl_for` - The list of transitions to inject default implementation for. The default value is `:all`.\n\n* `:timer` (`t:pos_integer/0`) - The interval to call `on_timer/2` recurrent event. The default value is `5000`.\n\n* `:auto_terminate` - When `true`, the transition to the end state is initiated automatically. The default value is `false`.\n\n* `:ensure_entry` - The list of states to retry transition to until succeeded. The default value is `[]`.\n\n* `:shutdown` (`t:pos_integer/0`) - The shutdown interval for the `GenServer` behind the FSM. The default value is `5000`.\n\n* `:persistency` - The implementation of `Finitomata.Persistency` behaviour to backup FSM with a persistent storage. The default value is `nil`.\n\n* `:listener` - The implementation of `Finitomata.Listener` behaviour _or_ a `GenServer.name()` to receive notification after transitions. The default value is `nil`.","ref":"Finitomata.html#module-options-to-use-finitomata"},{"type":"function","title":"Finitomata.alive?/2","doc":"Returns `true` if the _FSM_ specified is alive, `false` otherwise.","ref":"Finitomata.html#alive?/2"},{"type":"function","title":"Finitomata.allowed?/3","doc":"Returns `true` if the transition to the state `state` is possible, `false` otherwise.","ref":"Finitomata.html#allowed?/3"},{"type":"function","title":"Finitomata.fqn/2","doc":"Fully qualified name of the _FSM_ backed by `Finitonata`","ref":"Finitomata.html#fqn/2"},{"type":"function","title":"Finitomata.lookup/2","doc":"Fast check to validate the FSM process with such `id` and `target` exists.\n\nThe arguments are\n\n- the id of the FSM (optional)\n- the name of the FSM","ref":"Finitomata.html#lookup/2"},{"type":"callback","title":"Finitomata.on_enter/2","doc":"This callback will be called on entering the state.","ref":"Finitomata.html#c:on_enter/2"},{"type":"callback","title":"Finitomata.on_exit/2","doc":"This callback will be called on exiting the state.","ref":"Finitomata.html#c:on_exit/2"},{"type":"callback","title":"Finitomata.on_failure/3","doc":"This callback will be called if the transition failed to complete to allow\nthe consumer to take an action upon failure.","ref":"Finitomata.html#c:on_failure/3"},{"type":"callback","title":"Finitomata.on_start/1","doc":"This callback will be called from the underlying `c:GenServer.init/1`.\n\nUnlike other callbacks, this one might raise preventing the whole FSM from start.\n\nWhen `:ignore`, or `{:continues, new_payload}` tuple is returned from the callback,\n   the normal initalization continues through continuing to the next state.\n\n`{:ok, new_payload}` prevents the _FSM_ from automatically getting into start state,\n  and the respective transition must be called manually.","ref":"Finitomata.html#c:on_start/1"},{"type":"callback","title":"Finitomata.on_terminate/1","doc":"This callback will be called on transition to the final state to allow\nthe consumer to perform some cleanup, or like.","ref":"Finitomata.html#c:on_terminate/1"},{"type":"callback","title":"Finitomata.on_timer/2","doc":"This callback will be called recurrently if `timer: pos_integer()`\n  option has been given to `use Finitomata`.","ref":"Finitomata.html#c:on_timer/2"},{"type":"callback","title":"Finitomata.on_transition/4","doc":"This callback will be called from each transition processor.","ref":"Finitomata.html#c:on_transition/4"},{"type":"function","title":"Finitomata.responds?/3","doc":"Returns `true` if the transition by the event `event` is possible, `false` otherwise.","ref":"Finitomata.html#responds?/3"},{"type":"function","title":"Finitomata.start_fsm/4","doc":"Starts the FSM instance.\n\nThe arguments are\n\n- the global name of `Finitomata` instance (optional, defaults to `Finitomata`)\n- the name of the FSM (might be any term, but it must be unique)\n- the implementation of FSM (the module, having `use Finitomata`)\n- the payload to be carried in the FSM state during the lifecycle\n\nBefore `v0.15.0` the second and third parameters were expected in different order.\nThis is deprecated and will be removed in `v1.0.0`.\n\nThe FSM is started supervised. If the global name/id is given, it should be passed\n  to all calls like `transition/4`","ref":"Finitomata.html#start_fsm/4"},{"type":"function","title":"Finitomata.state/3","doc":"The state of the FSM.\n\nThe arguments are\n\n- the id of the FSM (optional)\n- the name of the FSM\n- defines whether the cached state might be returned or should be reloaded","ref":"Finitomata.html#state/3"},{"type":"function","title":"Finitomata.sup_alive?/1","doc":"Returns `true` if the supervision tree is alive, `false` otherwise.","ref":"Finitomata.html#sup_alive?/1"},{"type":"function","title":"Finitomata.sup_tree/1","doc":"Returns supervision tree of `Finitomata`. The healthy tree has all three `pid`s.","ref":"Finitomata.html#sup_tree/1"},{"type":"function","title":"Finitomata.transition/4","doc":"Initiates the transition.\n\nThe arguments are\n\n- the id of the FSM (optional)\n- the name of the FSM\n- `event` atom or `{event, event_payload}` tuple; the payload will be passed to the respective\n  `on_transition/4` call, payload is `nil` by default\n- `delay` (optional) the interval in milliseconds to apply transition after","ref":"Finitomata.html#transition/4"},{"type":"type","title":"Finitomata.event_payload/0","doc":"The payload that can be passed to each call to `transition/3`","ref":"Finitomata.html#t:event_payload/0"},{"type":"type","title":"Finitomata.fsm_name/0","doc":"The name of the FSM (might be any term, but it must be unique)","ref":"Finitomata.html#t:fsm_name/0"},{"type":"type","title":"Finitomata.id/0","doc":"The ID of the `Finitomata` supervision tree, useful for the concurrent\n  using of different `Finitomata` supervision trees.","ref":"Finitomata.html#t:id/0"},{"type":"type","title":"Finitomata.transition_resolution/0","doc":"The resolution of transition, when `{:error, _}` tuple, the transition is aborted","ref":"Finitomata.html#t:transition_resolution/0"},{"type":"type","title":"Finitomata.validation_error/0","doc":"Error types of FSM validation","ref":"Finitomata.html#t:validation_error/0"},{"type":"module","title":"Finitomata.ExUnit","doc":"Helpers and assertions to make `Finitomata` implementation easily testable.","ref":"Finitomata.ExUnit.html"},{"type":"macro","title":"Finitomata.ExUnit.assert_transition/3","doc":"Convenience macro to assert a transition initiated by `event_payload`\n  argument on the _FSM_ defined by the test context previously setup\n  with a call to `setup_finitomata/1`.\n\nLast regular argument in a call to `assert_transition/3` would be an\n  `event_payload` in a form of `{event, payload}`, or just `event`\n  for no payload.\n\n`to_state` argument would be matched to the resulting state of the transition,\n  and `block` accepts validation of the `payload` after transition in a form of\n\n```elixir\ntest \"some\", ctx do\n  assert_transition ctx, {:increase, 1} do\n    :counted ->\n      assert_payload do\n        user_data.counter ~> 2\n        internals.pid ~> ^parent\n      end\n      # or: assert_payload %{user_data: %{counter: 2}, internals: %{pid: ^parent}}\n\n      assert_receive {:increased, 2}\n  end\nend\n```","ref":"Finitomata.ExUnit.html#assert_transition/3"},{"type":"macro","title":"Finitomata.ExUnit.assert_transition/5","doc":"Convenience macro to assert a transition initiated by `event_payload`\n  argument on the _FSM_ defined by first three arguments.\n\n**NB** it’s not recommended to use low-level helpers, normally one should\n  define an _FSM_ in `setup_finitomata/1` block and use `assert_transition/3`\n  or even better `test_path/3`.\n\nLast regular argument in a call to `assert_transition/3` would be an\n  `event_payload` in a form of `{event, payload}`, or just `event`\n  for no payload.\n\n`to_state` argument would be matched to the resulting state of the transition,\n  and `block` accepts validation of the `payload` after transition in a form of\n\n```elixir\nparent = self()\n\nassert_transition id, impl, name, {:increase, 1} do\n  :counted ->\n    assert_payload do\n      user_data.counter ~> 2\n      internals.pid ~> ^parent\n    end\n    # or: assert_payload %{user_data: %{counter: 2}, internals: %{pid: ^parent}}\n    assert_receive {:increased, 2}\nend\n```","ref":"Finitomata.ExUnit.html#assert_transition/5"},{"type":"macro","title":"Finitomata.ExUnit.init_finitomata/5","doc":"This macro initiates the _FSM_ implementation specified by arguments passed.\n\n**NB** it’s not recommended to use low-level helpers, normally one should\n  define an _FSM_ in `setup_finitomata/1` block, which would initiate\n  the _FSM_ amongs other things.\n\n_Arguments:_\n\n- `id` — a `Finitomata` instance, carrying multiple _FSM_s\n- `impl` — the module implementing _FSM_ (having `use Finitomata` clause)\n- `name` — the name of the _FSM_\n- `payload` — the initial payload for this _FSM_\n- `options` — the options to control the test, such as\n  - `transition_count` — the number of expectations to declare (defaults to number of states)\n\nOnce called, this macro will start `Finitomata.Suprevisor` with the `id` given,\n  define a mox for `impl` unless already efined,\n  `Mox.allow/3` the _FSM_ to call testing process,\n  and expectations as a listener to `after_transition/3` callback,\n  sending a message of a shape `{:on_transition, id, state, payload}` to test process.\n\nThen it’ll start _FSM_ and ensure it has entered `Finitomata.Transition.entry/2` state.","ref":"Finitomata.ExUnit.html#init_finitomata/5"},{"type":"macro","title":"Finitomata.ExUnit.setup_finitomata/1","doc":"Setups `Finitomata` for testing in the case and/or in `ExUnit.Case.describe/2` block.\n\nIt would effectively init the _FSM_ with an underlying call to `init_finitomata/5`,\n  and put `finitomata` key into `context`, assigning `:test_pid` subkey to the `pid`\n  of the running test process, and mixing `:context` content into test context.\n\nAlthough one might pass the name, it’s more convenient to avoid doing it, in this case\n  the name would be assigned from the test name, which guarantees uniqueness of\n  _FSM_s running in concurrent environment.\n\nIt should return the keyword which would be validated with `NimbleOptions` schema\n\n* `:fsm` (non-empty `t:keyword/0`) - Required. The _FSM_ declaration to be used in tests.\n\n  * `:id` (`t:term/0`) - The ID of the `Finitomata` tree. The default value is `nil`.\n\n  * `:implementation` - Required. The implementatoin of `Finitomata` (the module with `use Finitomata`.)\n\n  * `:name` (`t:String.t/0`) - The name of the `Finitomata` instance.\n\n  * `:payload` (`t:term/0`) - Required. The initial payload for the _FSM_ to start with.\n\n  * `:options` (`t:keyword/0`) - Additional options to use in _FSM_ initialization. The default value is `[]`.\n\n    * `:transition_count` (`t:non_neg_integer/0`) - The expected by `Mox` number of transitions to handle.\n\n* `:context` (`t:keyword/0`) - The additional context to be passed to actual `ExUnit.Callbacks.setup/2` call.\n\n\n\n_Example:_\n\n```elixir\ndescribe \"MyFSM tests\" do\n  setup_finitomata do\n    parent = self()\n\n    [\n      fsm: [implementation: MyFSM, payload: %{}],\n      context: [parent: parent]\n    ]\n  end\n\n  …\n```","ref":"Finitomata.ExUnit.html#setup_finitomata/1"},{"type":"macro","title":"Finitomata.ExUnit.test_path/3","doc":"Convenience macro to test the whole _Finitomata_ path,\n  from starting to ending state.\n\nMust be used with a `setup_finitomata/1` callback.\n\n_Example:_\n\n```elixir\n  test_path \"The only path\", %{finitomata: %{test_pid: parent}} do\n    {:start, self()} ->\n      assert_state :started do\n        assert_payload do\n          internals.counter ~> 1\n          pid ~> ^parent\n        end\n\n        assert_receive {:on_start, ^parent}\n      end\n\n    :do ->\n      assert_state :done do\n        assert_receive :on_do\n      end\n\n      assert_state :* do\n        assert_receive :on_end\n      end\n  end\n```","ref":"Finitomata.ExUnit.html#test_path/3"},{"type":"module","title":"Finitomata.Pool","doc":"The instance of _FSM_ backed up by `Finitomata`.","ref":"Finitomata.Pool.html"},{"type":"module","title":"FSM representation - Finitomata.Pool","doc":"```mermaid\ngraph TD\n    idle --> |init| ready\n    idle --> |do| ready\n    ready --> |do| ready\n    ready --> |stop| done\n```\n\n---\nFully asynchronous pool to manage many similar processes, like connections.\n\nThe pool is to be started using `start_pool/1` directly or with `pool_spec/1` in the\n  supervision tree.\n\n`initialize/2` is explicitly separated because usually this is to be done after some\n  external service initialization. In a case of `AMQP` connection management, one\n  would probably start the connection process and then a pool to manage channels.\n\nOnce `initialize/2` has been called, the `run/3` function might be invoked to\n  asynchronously execute the function passed as `actor` to `start_pool/1`.\n\nIf the callbacks `on_result/2` and/or `on_error/2` are defined, they will be invoked\n  respectively. Finally, the message to the calling process will be sent, unless\n  the third argument in a call to `run/3` is `nil`.","ref":"Finitomata.Pool.html#module-fsm-representation"},{"type":"function","title":"Finitomata.Pool.__config__/1","doc":"Getter for the internal compiled-in _FSM_ information.","ref":"Finitomata.Pool.html#__config__/1"},{"type":"function","title":"Finitomata.Pool.__generator__/0","doc":"See `Finitomata.Pool.__generator__/1`","ref":"Finitomata.Pool.html#__generator__/0"},{"type":"function","title":"Finitomata.Pool.__generator__/1","doc":"Returns the generator to be used in `StreamData`-powered property testing, based\n  on the specification given to `use Estructura.Nested`, which contained","ref":"Finitomata.Pool.html#__generator__/1"},{"type":"function","title":"shape - Finitomata.Pool.__generator__/1","doc":"```elixir\n%{\n  id: {StreamData, :atom, [:alias]},\n  errors: [:term],\n  payload: :term,\n  actor: {StreamData, :constant, [&Function.identity/1]},\n  on_error: {Finitomata.Pool.Actor, :handler, [:error]},\n  on_result: {Finitomata.Pool.Actor, :handler, [[]]}\n}\n```\n\n\nThe argument given would be used as a template to generate new values.","ref":"Finitomata.Pool.html#__generator__/1-shape"},{"type":"function","title":"Finitomata.Pool.cast/2","doc":"Casts the map representation as given to `Estructura.Nested.shape/1` to\n  the nested `Estructura` instance.\n\nIf `split: true` is passed as an option, it will attempt to put `foo_bar` into nested `%{foo: %{bar: _}}`","ref":"Finitomata.Pool.html#cast/2"},{"type":"function","title":"Finitomata.Pool.cast!/2","doc":"","ref":"Finitomata.Pool.html#cast!/2"},{"type":"function","title":"Finitomata.Pool.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Finitomata.Pool.html#child_spec/1"},{"type":"macro","title":"Finitomata.Pool.config/1","doc":"The convenient macro to allow using states in guards, returns a compile-time\n  list of states for `Finitomata.Pool`.","ref":"Finitomata.Pool.html#config/1"},{"type":"function","title":"Finitomata.Pool.get/3","doc":"Gets the value for the given key from the structure","ref":"Finitomata.Pool.html#get/3"},{"type":"function","title":"Finitomata.Pool.initialize/2","doc":"Initializes the started _FSM_s with the same payload, or with what `payload_fun/1`\n  would return as many times as the number of workers.","ref":"Finitomata.Pool.html#initialize/2"},{"type":"function","title":"Finitomata.Pool.parse/1","doc":"Safely parses the json, applying all the specified validations and coercions","ref":"Finitomata.Pool.html#parse/1"},{"type":"function","title":"Finitomata.Pool.parse!/1","doc":"Same as `parse/1` but either returns the result of successful parsing or raises","ref":"Finitomata.Pool.html#parse!/1"},{"type":"function","title":"Finitomata.Pool.pool_spec/1","doc":"Child spec for `Finitomata.Pool` to embed the process into a supervision tree","ref":"Finitomata.Pool.html#pool_spec/1"},{"type":"function","title":"Finitomata.Pool.put/3","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns `{:ok, updated_struct}` or `{:error, reason}` if there is no such key","ref":"Finitomata.Pool.html#put/3"},{"type":"function","title":"Finitomata.Pool.put!/3","doc":"Puts the value for the given key into the structure, passing coercion _and_ validation,\n  returns the value or raises if there is no such key","ref":"Finitomata.Pool.html#put!/3"},{"type":"function","title":"Finitomata.Pool.run/3","doc":"The runner for the `actor` function with the specified payload.\n\nBasically, upon calling `run/3`, the following chain of calls would have happened:\n\n1. `actor.(payload, state)` (or `actor.(payload)` if the function of arity one had been given)\n2. `on_result(result, id)` / `on_error(result, id)` if callbacks are specified\n3. the message of the shape `{:transition, :success/:failure, self(), {payload, result, on_result/on_error}})` \n   will be sent to `pid` unless `nil` given as a third argument","ref":"Finitomata.Pool.html#run/3"},{"type":"function","title":"Finitomata.Pool.start_link/1","doc":"Starts an _FSM_ alone with `name` and `payload` given.\n\nUsually one does not want to call this directly, the most common way would be\nto start a `Finitomata` supervision tree or even better embed it into\nthe existing supervision tree _and_ start _FSM_ with `Finitomata.start_fsm/4`\npassing `Finitomata.Pool` as the first parameter.\n\nFor distributed applications, use `Infinitomata.start_fsm/4` instead.","ref":"Finitomata.Pool.html#start_link/1"},{"type":"function","title":"Finitomata.Pool.start_pool/1","doc":"Starts a pool of asynchronous workers wrapped by an _FSM_.","ref":"Finitomata.Pool.html#start_pool/1"},{"type":"function","title":"Finitomata.Pool.start_pool/3","doc":"","ref":"Finitomata.Pool.html#start_pool/3"},{"type":"type","title":"Finitomata.Pool.actor/0","doc":"The actor function in the pool","ref":"Finitomata.Pool.html#t:actor/0"},{"type":"type","title":"Finitomata.Pool.handler/0","doc":"The handler function in the pool","ref":"Finitomata.Pool.html#t:handler/0"},{"type":"type","title":"Finitomata.Pool.id/0","doc":"The ID of the Pool","ref":"Finitomata.Pool.html#t:id/0"},{"type":"type","title":"Finitomata.Pool.naive_actor/0","doc":"The simple actor function in the pool","ref":"Finitomata.Pool.html#t:naive_actor/0"},{"type":"type","title":"Finitomata.Pool.naive_handler/0","doc":"The simple handler of result/error in the pool","ref":"Finitomata.Pool.html#t:naive_handler/0"},{"type":"type","title":"Finitomata.Pool.responsive_actor/0","doc":"The actor function in the pool, receiving the state as a second argument","ref":"Finitomata.Pool.html#t:responsive_actor/0"},{"type":"type","title":"Finitomata.Pool.responsive_handler/0","doc":"The actor function in the pool, receiving the state as a second argument","ref":"Finitomata.Pool.html#t:responsive_handler/0"},{"type":"type","title":"Finitomata.Pool.state/0","doc":"Kind of event which might be send to initiate the transition.","ref":"Finitomata.Pool.html#t:state/0"},{"type":"type","title":"FSM representation - Finitomata.Pool.state/0","doc":"```mermaid\ngraph TD\n    idle --> |init| ready\n    idle --> |do| ready\n    ready --> |do| ready\n    ready --> |stop| done\n```","ref":"Finitomata.Pool.html#t:state/0-fsm-representation"},{"type":"type","title":"Finitomata.Pool.t/0","doc":"Type of this struct","ref":"Finitomata.Pool.html#t:t/0"},{"type":"module","title":"Finitomata.Throttler","doc":"The internal definition of the call to throttle.\n\n`Finitomata.Throttler.call/3` is a blocking call similar to `GenServer.call/3`, but\n  served by the underlying `GenStage` producer-consumer pair.\n\nDespite this implementation of throttling based on `GenStage` is provided\n  mostly for internal needs, it is generic enough to use wherever. Use the childspec\n  `{Finitomata.Throttler, name: name, initial: [], max_demand: 3, interval: 1_000}`\n  to start a throttling process and `Finitomata.Throttler.call/3` to perform throttled\n  synchronous calls from different processes.\n\n#","ref":"Finitomata.Throttler.html"},{"type":"module","title":"Usage - Finitomata.Throttler","doc":"```elixir\n{:ok, pid} = Finitomata.Throttler.start_link(name: Throttler)\n\nFinitomata.Throttler.call(Throttler, {IO, :inspect, [42]})\n42\n\n#⇒ %Finitomata.Throttler{\n#   from: {#PID<0.335.0>, #Reference<0.3154300821.2643722246.59214>},\n#   fun: {IO, :inspect},\n#   args: ~c\"*\",\n#   result: 42,\n#   duration: 192402,\n#   payload: nil\n# }  \n```","ref":"Finitomata.Throttler.html#module-usage"},{"type":"function","title":"Finitomata.Throttler.call/3","doc":"Synchronously executes the function, using throttling based on `GenStage`.\n\nThis function has a default timeout `:infinity` because of its nature\n  (throttling is supposed to take a while,) but it might be passed as the third\n  argument in a call to `call/3`.\n\nIf a list of functions is given, executes all of them in parallel,\n  collects the results, and then returns them to the caller.\n\nThe function might be given as `t:Finitomata.Throttler.t/0` or\n  in a simplified form as `{function_of_arity_1, arg}` or `{mod, fun, args}`.","ref":"Finitomata.Throttler.html#call/3"},{"type":"function","title":"Finitomata.Throttler.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Finitomata.Throttler.html#child_spec/1"},{"type":"function","title":"Finitomata.Throttler.start_link/1","doc":"Starts the throttler with the underlying producer-consumer stages.\n\nAccepted options are:\n\n- `name` the base name for the throttler to be used in calls to `call/3`\n- `initial` the initial load of requests (avoid using it unless really needed)\n- `max_demand`, `initial` the options to be passed directly to `GenStage`’s consumer","ref":"Finitomata.Throttler.html#start_link/1"},{"type":"type","title":"Finitomata.Throttler.t/0","doc":"The _in/out_ parameter for calls to `Finitomata.Throttler.call/3`","ref":"Finitomata.Throttler.html#t:t/0"},{"type":"type","title":"Finitomata.Throttler.throttlee/0","doc":"The simplified _in_ parameter for calls to `Finitomata.Throttler.call/3`","ref":"Finitomata.Throttler.html#t:throttlee/0"},{"type":"module","title":"Infinitomata","doc":"The sibling of `Finitomata`, but runs transparently in the cluster.\n\nIf you want to use a _stateful consistent hash ring_ like [`libring`](https://hexdocs.pm/libring),\n  implement the behaviour `Finitomata.ClusterInfo` wrapping calls to it and \n  invoke `Finitomata.ClusterInfo.init(Impl)` before using `Infinitomata.start_fsm/4`.\n\nThe example of such an implementation for `libring` (assuming the named ring `@ring`\n  has been started in the supervision tree) follows.\n\n```elixir\ndefmodule MyApp.ClusterInfo do\n  @moduledoc false\n  @behaviour Finitomata.ClusterInfo\n\n  @impl Finitomata.ClusterInfo\n  def nodes, do: HashRing.nodes(@ring)\n\n  @impl Finitomata.ClusterInfo\n  def whois(id), do: HashRing.key_to_node(@ring, id) \nend\n```","ref":"Infinitomata.html"},{"type":"function","title":"Infinitomata.all/1","doc":"The full state with all te children, might be a heavy map","ref":"Infinitomata.html#all/1"},{"type":"function","title":"Infinitomata.child_spec/1","doc":"","ref":"Infinitomata.html#child_spec/1"},{"type":"function","title":"Infinitomata.count/1","doc":"Count of children","ref":"Infinitomata.html#count/1"},{"type":"function","title":"Infinitomata.random/1","doc":"Returns the random _FSM_ from the pool","ref":"Infinitomata.html#random/1"},{"type":"function","title":"Infinitomata.start_fsm/4","doc":"Starts the FSM somewhere in the cluster.\n\nSee `Finitomata.start_fsm/4`.","ref":"Infinitomata.html#start_fsm/4"},{"type":"function","title":"Infinitomata.start_link/1","doc":"","ref":"Infinitomata.html#start_link/1"},{"type":"function","title":"Infinitomata.state/3","doc":"The state of the FSM in the cluster.\n\nSee `Finitomata.state/3`.","ref":"Infinitomata.html#state/3"},{"type":"function","title":"Infinitomata.synch/1","doc":"Synchronizes the local `Infinitomata` instance with the cluster","ref":"Infinitomata.html#synch/1"},{"type":"function","title":"Infinitomata.transition/4","doc":"Initiates the transition in the cluster.\n\nSee `Finitomata.transition/4`.","ref":"Infinitomata.html#transition/4"},{"type":"behaviour","title":"Finitomata.ClusterInfo","doc":"The behaviour to be implemented for locating the node across the cluster.\n\n`Infinitomata` comes with a default naïve implementation, which simply responds\n  with a determined random value for `whois/1` and with all visible nodes list\n  for `nodes/0`.\n\nThe call to `Finitomata.ClusterInfo.init/1` passing the module implementing\n  the desired behaviour is mandatory before any call to `Infinitomata.start_fsm/4`\n  to preserve a determined consistency.","ref":"Finitomata.ClusterInfo.html"},{"type":"function","title":"Finitomata.ClusterInfo.init/1","doc":"Call this function to select an implementation of a cluster lookup","ref":"Finitomata.ClusterInfo.html#init/1"},{"type":"function","title":"Finitomata.ClusterInfo.nodes/0","doc":"Delegates to the selected implementation of a cluster lookup","ref":"Finitomata.ClusterInfo.html#nodes/0"},{"type":"callback","title":"Finitomata.ClusterInfo.nodes/0","doc":"Returns nodes available to select from","ref":"Finitomata.ClusterInfo.html#c:nodes/0"},{"type":"function","title":"Finitomata.ClusterInfo.whois/1","doc":"Delegates to the selected implementation of a cluster lookup","ref":"Finitomata.ClusterInfo.html#whois/1"},{"type":"callback","title":"Finitomata.ClusterInfo.whois/1","doc":"Returns the node “selected” for this particular `id`","ref":"Finitomata.ClusterInfo.html#c:whois/1"},{"type":"behaviour","title":"Finitomata.Listener","doc":"The behaviour to be implemented and passed to `use Finitomata` to receive\n  all the state transitions notifications.","ref":"Finitomata.Listener.html"},{"type":"callback","title":"Finitomata.Listener.after_transition/3","doc":"To be called after a successful transition","ref":"Finitomata.Listener.html#c:after_transition/3"},{"type":"behaviour","title":"Finitomata.Parser","doc":"The behaviour, defining the parser to produce _FSM_ out of textual representation.","ref":"Finitomata.Parser.html"},{"type":"callback","title":"Finitomata.Parser.lint/1","doc":"Linter of the input, producing a well-formed representation of _FSM_ understood by JS/markdown parsers.","ref":"Finitomata.Parser.html#c:lint/1"},{"type":"callback","title":"Finitomata.Parser.parse/1","doc":"Parse function, producing the _FSM_ definition as a list of `Transition` instances.","ref":"Finitomata.Parser.html#c:parse/1"},{"type":"callback","title":"Finitomata.Parser.validate/1","doc":"Validation of the input.","ref":"Finitomata.Parser.html#c:validate/1"},{"type":"type","title":"Finitomata.Parser.parse_error/0","doc":"The type representing the parsing error which might be passed through to raised `CompileError`.","ref":"Finitomata.Parser.html#t:parse_error/0"},{"type":"behaviour","title":"Finitomata.Pool.Actor","doc":"The behaviour specifying the actor in the pool.","ref":"Finitomata.Pool.Actor.html"},{"type":"callback","title":"Finitomata.Pool.Actor.actor/2","doc":"The function which would be invoked in `Finitomata.Pool.run/3`,\n  see `t:Finitomata.Pool.responsive_actor/0`","ref":"Finitomata.Pool.Actor.html#c:actor/2"},{"type":"function","title":"Finitomata.Pool.Actor.handler/0","doc":"`StreamData` helper to produce `on_error/2` and `on_result/2` handlers","ref":"Finitomata.Pool.Actor.html#handler/0"},{"type":"function","title":"Finitomata.Pool.Actor.handler/1","doc":"","ref":"Finitomata.Pool.Actor.html#handler/1"},{"type":"callback","title":"Finitomata.Pool.Actor.on_error/2","doc":"The function which would be invoked in `Finitomata.Pool.run/3`\n  after `actor/2` failed with a failure message and the state\n  of the `Pool` worker,\n  see `t:Finitomata.Pool.naive_handler/0`.\n\nThe value returned from this call will be send as an last argument \n  of the message to the caller of `Finitomata.Pool.run/3` if the `pid`\n  was passed.","ref":"Finitomata.Pool.Actor.html#c:on_error/2"},{"type":"callback","title":"Finitomata.Pool.Actor.on_result/2","doc":"The function which would be invoked in `Finitomata.Pool.run/3`\n  after `actor/2` returned successfully with a result of invocation\n  and the state of the `Pool` worker,\n  see `t:Finitomata.Pool.naive_handler/0`.\n\nThe value returned from this call will be send as an last argument \n  of the message to the caller of `Finitomata.Pool.run/3` if the `pid`\n  was passed.","ref":"Finitomata.Pool.Actor.html#c:on_result/2"},{"type":"module","title":"Finitomata.State","doc":"Carries the state of the FSM.","ref":"Finitomata.State.html"},{"type":"function","title":"Finitomata.State.excerpt/2","doc":"Exposes the short excerpt from state of FSM which is log-friendly","ref":"Finitomata.State.html#excerpt/2"},{"type":"type","title":"Finitomata.State.last_error/0","doc":"The map that holds last error which happened on transition (at given state and event).","ref":"Finitomata.State.html#t:last_error/0"},{"type":"type","title":"Finitomata.State.payload/0","doc":"The payload that has been passed to the FSM instance on startup","ref":"Finitomata.State.html#t:payload/0"},{"type":"type","title":"Finitomata.State.t/0","doc":"The internal representation of the FSM state","ref":"Finitomata.State.html#t:t/0"},{"type":"module","title":"Finitomata.Transition","doc":"The internal representation of `Transition`.\n\nIt includes `from` and `to` states, and the `event`, all represented as atoms.","ref":"Finitomata.Transition.html"},{"type":"function","title":"Finitomata.Transition.allowed/2","doc":"Returns the list of all the transitions, matching the options.\n\nUsed internally for the validations.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.Mermaid.parse(\n    ...>     \"idle --> |to_s1| s1\\n\" <>\n    ...>     \"s1 --> |to_s2| s2\\n\" <>\n    ...>     \"s1 --> |to_s3| s3\\n\" <>\n    ...>     \"s2 --> |to_s3| s3\")\n    ...> Finitomata.Transition.allowed(transitions, to: [:idle, :*])\n    [{:*, :idle, :__start__}, {:s3, :*, :__end__}]\n    iex> Finitomata.Transition.allowed(transitions, from: :s1)\n    [{:s1, :s2, :to_s2}, {:s1, :s3, :to_s3}]\n    iex> Finitomata.Transition.allowed(transitions, from: :s1, to: :s3)\n    [{:s1, :s3, :to_s3}]\n    iex> Finitomata.Transition.allowed(transitions, from: :s1, with: :to_s3)\n    [{:s1, :s3, :to_s3}]\n    iex> Finitomata.Transition.allowed(transitions, from: :s2, with: :to_s2)\n    []","ref":"Finitomata.Transition.html#allowed/2"},{"type":"function","title":"Finitomata.Transition.allowed/3","doc":"Returns the list of all the transitions, matching the `from` state and the `event`.\n\nUsed internally for the validations.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> s1 : foo\\ns1 --> s2 : ok\\ns2 --> [*] : ko\")\n    ...> Finitomata.Transition.allowed(transitions, :s1, :foo)\n    [:s2]\n    ...> Finitomata.Transition.allowed(transitions, :s1, :*)\n    []","ref":"Finitomata.Transition.html#allowed/3"},{"type":"function","title":"Finitomata.Transition.allowed?/3","doc":"Returns `true` if the transition `from` → `to` is allowed, `false` otherwise.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> s1 : foo\\ns1 --> s2 : ok\\ns2 --> [*] : ko\")\n    ...> Finitomata.Transition.allowed?(transitions, :s1, :s2)\n    true\n    ...> Finitomata.Transition.allowed?(transitions, :s1, :*)\n    false","ref":"Finitomata.Transition.html#allowed?/3"},{"type":"function","title":"Finitomata.Transition.ambiguous/1","doc":"Returns keyword list of\n  `{Finitomata.Transition.state(), [Finitomata.Transition.event()]}` for transitions\n  which do not have a determined _to_ state.\n\nUsed internally for the validations.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.Mermaid.parse(\n    ...>     \"idle --> |to_s1| s1\\n\" <>\n    ...>     \"s1 --> |to_s2| s2\\n\" <>\n    ...>     \"s1 --> |to_s3| s3\\n\" <>\n    ...>     \"s2 --> |to_s1| s3\\n\" <>\n    ...>     \"s2 --> |ambiguous| s3\\n\" <>\n    ...>     \"s2 --> |ambiguous| s4\\n\" <>\n    ...>     \"s3 --> |determined| s3\\n\" <>\n    ...>     \"s3 --> |determined| s4\\n\")\n    ...> Finitomata.Transition.ambiguous(transitions)\n    [s3: {:determined, [:s3, :s4]}, s2: {:ambiguous, [:s3, :s4]}]","ref":"Finitomata.Transition.html#ambiguous/1"},{"type":"function","title":"Finitomata.Transition.continuation/3","doc":"Returns the continuation from the state given which inevitably lead to other state(s).\n\nAll the transitions from this state are hard (ending with `!`,)\n  which makes the _FSM_ to go through all these states in `:continue` callbacks.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> entry : start\\nentry --> exit : go!\\nexit --> done : finish\\ndone --> [*] : terminate\")\n    ...> Finitomata.Transition.continuation(:entry, Finitomata.Transition.hard(:transitions, transitions))\n    %Finitomata.Transition.Path{from: :entry, to: :exit, path: [go!: :exit]}","ref":"Finitomata.Transition.html#continuation/3"},{"type":"function","title":"Finitomata.Transition.determined/1","doc":"Returns keyword list of\n  `{Finitomata.Transition.state(), Finitomata.Transition.event()}` tuples\n  for determined transition from the current state.\n\nThe transition is determined, if it is the only transition allowed from the state.\n\nUsed internally for the validations.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.Mermaid.parse(\n    ...>     \"idle --> |to_s1| s1\\n\" <>\n    ...>     \"s1 --> |to_s2| s2\\n\" <>\n    ...>     \"s1 --> |to_s3| s3\\n\" <>\n    ...>     \"s2 --> |to_s1| s3\\n\" <>\n    ...>     \"s2 --> |ambiguous| s3\\n\" <>\n    ...>     \"s2 --> |ambiguous| s4\\n\" <>\n    ...>     \"s3 --> |determined| s3\\n\" <>\n    ...>     \"s3 --> |determined| s4\\n\")\n    ...> Finitomata.Transition.determined(transitions)\n    [s4: :__end__, s3: :determined, idle: :to_s1]","ref":"Finitomata.Transition.html#determined/1"},{"type":"function","title":"Finitomata.Transition.determined/2","doc":"Returns `{:ok, {event(), state()}}` tuple if there is a determined transition\n  from the current state, `:error` otherwise.\n\nThe transition is determined, if it is the only transition allowed from the state.\n\nUsed internally for the validations.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.Mermaid.parse(\n    ...>     \"idle --> |to_s1| s1\\n\" <>\n    ...>     \"s1 --> |to_s2| s2\\n\" <>\n    ...>     \"s1 --> |to_s3| s3\\n\" <>\n    ...>     \"s2 --> |to_s3| s3\")\n    ...> Finitomata.Transition.determined(transitions, :s1)\n    :error\n    iex> Finitomata.Transition.determined(transitions, :s2)\n    {:ok, {:to_s3, :s3}}\n    iex> Finitomata.Transition.determined(transitions, :s3)\n    {:ok, {:__end__, :*}}","ref":"Finitomata.Transition.html#determined/2"},{"type":"function","title":"Finitomata.Transition.entry/2","doc":"Returns the state _after_ starting one, so-called `entry` state.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> entry : foo\\nentry --> exit : go\\nexit --> [*] : terminate\")\n    ...> Finitomata.Transition.entry(transitions)\n    :entry","ref":"Finitomata.Transition.html#entry/2"},{"type":"function","title":"Finitomata.Transition.event_kind/1","doc":"Returns the kind of event.\n\nIf event ends up with an exclamation sign, it’s `:hard`, meaning the respective\n  transition would be initiated automatically when the `from` state of such a transition\n  is reached.\n\nIf event ends up with a question mark, it’s `:soft`, meaning no error would have\n  been reported in a case transition fails.\n\nOtherwise the event is `:normal`.\n\n    iex> {:ok, [_, hard, _]} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> entry : foo\\nentry --> exit : go!\\nexit --> [*] : terminate\")\n    ...> Finitomata.Transition.event_kind(hard)\n    :hard","ref":"Finitomata.Transition.html#event_kind/1"},{"type":"function","title":"Finitomata.Transition.events/2","doc":"Returns the not ordered list of events, including or excluding\n  the internal starting and ending transitions `:__start__` and `__end__`\n  according to the second argument.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.Mermaid.parse(\"s1 --> |ok| s2\\ns1 --> |ko| s3\")\n    ...> Finitomata.Transition.events(transitions, true)\n    [:ok, :ko]\n    ...> Finitomata.Transition.events(transitions)\n    [:__start__, :ok, :ko, :__end__]","ref":"Finitomata.Transition.html#events/2"},{"type":"function","title":"Finitomata.Transition.exit/2","doc":"Returns the states _before_ ending one, so-called `exit` states.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.Mermaid.parse(\n    ...>     \"entry --> |process| processing\\nprocessing --> |ok| success\\nprocessing --> |ko| error\"\n    ...>   )\n    ...> Finitomata.Transition.exit(transitions)\n    [:error, :success]","ref":"Finitomata.Transition.html#exit/2"},{"type":"function","title":"Finitomata.Transition.exiting/2","doc":"Returns all the states which inevitably lead to the ending one.\n\nAll the transitions from these states to the ending one are hard (ending with `!`,)\n  which makes the _FSM_ to go through all these states in `:continue` callbacks.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> entry : start\\nentry --> exit : go!\\nexit --> [*] : terminate\")\n    ...> Finitomata.Transition.exiting(transitions)\n    [%Finitomata.Transition.Path{from: :entry, to: :*, path: [go!: :exit, terminate: :*]}]","ref":"Finitomata.Transition.html#exiting/2"},{"type":"function","title":"Finitomata.Transition.hard/2","doc":"Returns all the hard transitions which inevitably lead to the next state\n  (events ending with an exclamation sign,)\n  which makes the _FSM_ to go to the next state with `:continue` callback.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> entry : start\\nentry --> exit : go!\\nexit --> [*] : terminate\")\n    ...> Finitomata.Transition.hard(transitions)\n    [entry: :go!]\n    ...> Finitomata.Transition.hard(:transitions, transitions)\n    [%Finitomata.Transition{from: :entry, to: :exit, event: :go!}]","ref":"Finitomata.Transition.html#hard/2"},{"type":"function","title":"Finitomata.Transition.loops/2","doc":"Returns all the loops aka internal paths where starting and ending states are the same one.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> s1 : foo\\ns1 --> s2 : ok\\ns2 --> s1 : ok\\ns2 --> [*] : ko\")\n    ...> Finitomata.Transition.loops(transitions)\n    [%Finitomata.Transition.Path{from: :s1, to: :s1, path: [ok: :s2, ok: :s1]},\n     %Finitomata.Transition.Path{from: :s2, to: :s2, path: [ok: :s1, ok: :s2]}]","ref":"Finitomata.Transition.html#loops/2"},{"type":"function","title":"Finitomata.Transition.paths/4","doc":"Returns all the paths from starting to ending state.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> s1 : foo\\ns1 --> s2 : ok\\ns1 --> s3 : ok\\ns2 --> [*] : ko\\ns3 --> [*] : ko\")\n    ...> Finitomata.Transition.paths(transitions)\n    [%Finitomata.Transition.Path{from: :*, to: :*, path: [foo: :s1, ok: :s2, ko: :*]},\n     %Finitomata.Transition.Path{from: :*, to: :*, path: [foo: :s1, ok: :s3, ko: :*]}]","ref":"Finitomata.Transition.html#paths/4"},{"type":"function","title":"Finitomata.Transition.responds?/3","doc":"Returns `true` if the state `from` hsa an outgoing transition with `event`, false otherwise.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> s1 : foo\\ns1 --> s2 : ok\\ns2 --> [*] : ko\")\n    ...> Finitomata.Transition.responds?(transitions, :s1, :ok)\n    true\n    ...> Finitomata.Transition.responds?(transitions, :s1, :ko)\n    false","ref":"Finitomata.Transition.html#responds?/3"},{"type":"function","title":"Finitomata.Transition.states/2","doc":"Returns the not ordered list of states, including or excluding\n  the starting and ending states `:*` according to the second argument.\n\n    iex> {:ok, transitions} =\n    ...>   Finitomata.PlantUML.parse(\"[*] --> s1 : foo\\ns1 --> s2 : ok\\ns2 --> [*] : ko\")\n    ...> Finitomata.Transition.states(transitions, true)\n    [:s1, :s2]\n    ...> Finitomata.Transition.states(transitions)\n    [:*, :s1, :s2]","ref":"Finitomata.Transition.html#states/2"},{"type":"type","title":"Finitomata.Transition.event/0","doc":"The event in FSM","ref":"Finitomata.Transition.html#t:event/0"},{"type":"type","title":"Finitomata.Transition.event_kind/0","doc":"The kind of event","ref":"Finitomata.Transition.html#t:event_kind/0"},{"type":"type","title":"Finitomata.Transition.state/0","doc":"The state of FSM","ref":"Finitomata.Transition.html#t:state/0"},{"type":"type","title":"Finitomata.Transition.t/0","doc":"The transition is represented by `from` and `to` states _and_ the `event`.","ref":"Finitomata.Transition.html#t:t/0"},{"type":"module","title":"Finitomata.Transition.Path","doc":"The path from one state to another one","ref":"Finitomata.Transition.Path.html"},{"type":"type","title":"Finitomata.Transition.Path.t/0","doc":"","ref":"Finitomata.Transition.Path.html#t:t/0"},{"type":"behaviour","title":"Finitomata.Persistency","doc":"The behaviour to be implemented by a persistent storage to be used\n  with `Finitomata` (pass the implementation as `persistency: Impl.Module.Name`\n  to `use Finitomata`.)\n\nOnce declared, the initial state would attempt to load the current state from\n  the storage using `load/1` funtcion which should return the `{state, payload}`\n  tuple.","ref":"Finitomata.Persistency.html"},{"type":"callback","title":"Finitomata.Persistency.load/1","doc":"The function to be called from `init/1` callback upon FSM start to load the state and payload\n  from the persistent storage","ref":"Finitomata.Persistency.html#c:load/1"},{"type":"callback","title":"Finitomata.Persistency.store/3","doc":"The function to be called from `on_transition/4` handler to allow storing the state\n  and payload to the persistent storage","ref":"Finitomata.Persistency.html#c:store/3"},{"type":"callback","title":"Finitomata.Persistency.store_error/4","doc":"The function to be called from `on_transition/4` handler on non successful\n  transition to allow storing the failed attempt to transition to the persistent storage","ref":"Finitomata.Persistency.html#c:store_error/4"},{"type":"type","title":"Finitomata.Persistency.transition_info/0","doc":"","ref":"Finitomata.Persistency.html#t:transition_info/0"},{"type":"protocol","title":"Finitomata.Persistency.Persistable","doc":"The protocol to be implemented for custom data to be used in pair with\n  `Finitomata.Persistency.Protocol` persistency adapter.\n\nFor that combination, one should implement the protocol for that particular\n  struct _and_ specify `Finitomata.Persistency.Protocol` as a `persistency`\n  in a call to `use Finitomata`.\n\n```elixir\nuse Finitomata, …, persistency: Finitomata.Persistency.Protocol\n```","ref":"Finitomata.Persistency.Persistable.html"},{"type":"function","title":"Finitomata.Persistency.Persistable.load/1","doc":"Loads the entity from some external storage","ref":"Finitomata.Persistency.Persistable.html#load/1"},{"type":"function","title":"Finitomata.Persistency.Persistable.store/2","doc":"Persists the transitioned entity to some external storage","ref":"Finitomata.Persistency.Persistable.html#store/2"},{"type":"function","title":"Finitomata.Persistency.Persistable.store_error/3","doc":"Persists the error happened while an attempt to transition the entity","ref":"Finitomata.Persistency.Persistable.html#store_error/3"},{"type":"type","title":"Finitomata.Persistency.Persistable.t/0","doc":"All the types that implement this protocol.","ref":"Finitomata.Persistency.Persistable.html#t:t/0"},{"type":"module","title":"Finitomata.Persistency.Protocol","doc":"Default implementation of persistency adapter that does nothing but routes\n  to the implementation of `Finitomata.Persistency.Persistable` for the data.","ref":"Finitomata.Persistency.Protocol.html"},{"type":"function","title":"Finitomata.Persistency.Protocol.idfy/1","doc":"","ref":"Finitomata.Persistency.Protocol.html#idfy/1"},{"type":"exception","title":"Finitomata.TestTransitionError","doc":"","ref":"Finitomata.TestTransitionError.html"},{"type":"extras","title":"Finitomata","doc":"# ![Finitomata](https://raw.githubusercontent.com/am-kantox/finitomata/master/stuff/finitomata-48x48.png) Finitomata    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/finitomata/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/finitomata/workflows/Dialyzer/badge.svg)\n\n**The FSM boilerplate based on callbacks**\n\n---","ref":"readme.html"},{"type":"extras","title":"Bird View - Finitomata","doc":"`Finitomata` provides a boilerplate for [FSM](https://en.wikipedia.org/wiki/Finite-state_machine) implementation, allowing to concentrate on the business logic rather than on the process management and transitions/events consistency tweaking.\n\nIt reads a description of the FSM from a string in [PlantUML](https://plantuml.com/en/state-diagram), [Mermaid](https://mermaid.live), or even custom format. \n\n> #","ref":"readme.html#bird-view"},{"type":"extras","title":"Syntax Definition {: .tip} - Finitomata","doc":">\n> `Mermaid` **state diagram** format is literally the same as `PlantUML`, so if you want to use it, specify `syntax: :state_diagram` and\n> if you want to use **mermaid graph**, specify `syntax: :flowchart`. The latter is the default.\n\nBasically, it looks more or less like this\n\n### `PlantUML` / `:state_diagram`\n\n    [*] --> s1 : to_s1\n    s1 --> s2 : to_s2\n    s1 --> s3 : to_s3\n    s2 --> [*] : ok\n    s3 --> [*] : ok\n\n### `Mermaid` / `:flowchart`\n\n    s1 --> |to_s2| s2\n    s1 --> |to_s3| s3\n\n> #","ref":"readme.html#syntax-definition-tip"},{"type":"extras","title":"Using `syntax: :flowchart` {: .tip} - Finitomata","doc":">\n> `Mermaid` does not allow to explicitly specify transitions (and hence event names)\n> from the starting state and to the end state(s), these states names are implicitly set to `:*`\n> and events to `:__start__` and `:__end__` respectively.\n\n`Finitomata` validates the FSM is consistent, namely it has a single initial state, one or more final states, and no orphan states. If everything is OK, it generates a `GenServer` that could be used both alone, and with provided supervision tree. This `GenServer` requires to implement six callbacks\n\n- `on_transition/4` — **mandatory**\n- `on_failure/3` — optional\n- `on_enter/2` — optional\n- `on_exit/2` — optional\n- `on_terminate/1` — optional\n- `on_timer/2` — optional\n\nAll the callbacks do have a default implementation, that would perfectly handle transitions having a single _to_ state and not requiring any additional business logic attached.\n\nUpon start, it moves to the next to initial state and sits there awaiting for the _transition request_. Then it would call an `on_transition/4` callback and move to the next state, or remain in the current one, according to the response.\n\nUpon reaching a final state, it would terminate itself. The process keeps all the history of states it went through, and might have a payload in its state.","ref":"readme.html#using-syntax-flowchart-tip"},{"type":"extras","title":"Special Events - Finitomata","doc":"If the event name is ended with a bang (e. g. `idle --> |start!| started`) _and_\nthis event is the only one allowed from this state (there might be several transitions though,)\nit’d be considered as _determined_ and FSM will be transitioned into the new state instantly.\n\nIf the event name is ended with a question mark (e. g. `idle --> |start?| started`,)\nthe transition is considered as expected to fail; no `on_failure/2` callback would\nbe called on failure and no log warning will be printed.","ref":"readme.html#special-events"},{"type":"extras","title":"FSM Tuning and Configuration - Finitomata","doc":"#","ref":"readme.html#fsm-tuning-and-configuration"},{"type":"extras","title":"Recurrent Callback - Finitomata","doc":"If `timer: non_neg_integer()` option is passed to `use Finitomata`,\nthen `c:Finitomata.on_timer/2` callback will be executed recurrently.\nThis might be helpful if _FSM_ needs to update its state from the outside\nworld on regular basis.\n\n#","ref":"readme.html#recurrent-callback"},{"type":"extras","title":"Automatic FSM Termination - Finitomata","doc":"If `auto_terminate: true() | state() | [state()]` option is passed to `use Finitomata`,\nthe special `__end__` event to transition to the end state will be called automatically\nunder the hood, if the current state is either listed explicitly, or if the value of\nthe parameter is `true`.\n\n#","ref":"readme.html#automatic-fsm-termination"},{"type":"extras","title":"Ensuring State Entry - Finitomata","doc":"If `ensure_entry: true() | [state()]` option is passed to `use Finitomata`, the transition\nattempt will be retried with `{:continue, {:transition, {event(), event_payload()}}}` message\nuntil succeeded. Neither `on_failure/2` callback is called nor warning message is logged.\n\nThe payload would be updated to hold `__retries__: pos_integer()` key. If the payload was not a map,\nit will be converted to a map `%{payload: payload}`.","ref":"readme.html#ensuring-state-entry"},{"type":"extras","title":"Example - Finitomata","doc":"Let’s define the FSM instance\n\n```elixir\ndefmodule MyFSM do\n  @fsm \"\"\"\n  s1 --> |to_s2| s2\n  s1 --> |to_s3| s3\n  \"\"\"\n  use Finitomata, fsm: @fsm, syntax: :flowchart","ref":"readme.html#example"},{"type":"extras","title":"or uncomment lines below for `:state_diagram` syntax - Finitomata","doc":"# @fsm \"\"\"\n  # [*] --> s1 : to_s1\n  # s1 --> s2 : to_s2\n  # s1 --> s3 : to_s3\n  # s2 --> [*] : __end__\n  # s3 --> [*] : __end__\n  # \"\"\"\n  # use Finitomata, fsm: @fsm, syntax: :state_diagram\n\n  @impl Finitomata\n  def on_transition(:s1, :to_s2, _event_payload, state_payload),\n    do: {:ok, :s2, state_payload}\nend\n```\n\nNow we can play with it a bit.\n\n```elixir\n# or embed into supervision tree using `Finitomata.child_spec()`\n{:ok, _pid} = Finitomata.start_link()\n\nFinitomata.start_fsm MyFSM, \"My first FSM\", %{foo: :bar}\nFinitomata.transition \"My first FSM\", {:to_s2, nil}\nFinitomata.state \"My first FSM\"                    \n#⇒ %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}}\n\nFinitomata.allowed? \"My first FSM\", :* # state\n#⇒ true\nFinitomata.responds? \"My first FSM\", :to_s2 # event\n#⇒ false\n\nFinitomata.transition \"My first FSM\", {:__end__, nil} # to final state\n#⇒ [info]  [◉ ⇄] [state: %Finitomata.State{current: :s2, history: [:s1], payload: %{foo: :bar}}]\n\nFinitomata.alive? \"My first FSM\"\n#⇒ false\n```\n\nTypically, one would implement all the `on_transition/4` handlers, pattern matching on the state/event.\n\n---","ref":"readme.html#or-uncomment-lines-below-for-state_diagram-syntax"},{"type":"extras","title":"Installation - Finitomata","doc":"```elixir\ndef deps do\n  [\n    {:finitomata, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Changelog - Finitomata","doc":"- `0.19.0` — [UPD] `Finitomata.ExUnit` lighten options check (compile-time module dependencies suck in >=1.16) \n- `0.18.0` — [UPD] asynchronous `Finitomata.Pool` on top of `Infinitomata` \n- `0.17.0` — [UPD] careful naming and `Finitomata.Throttler`\n- `0.16.0` — [UPD] `Infinitomata` as a self-contained distributed implementation leveraging `:pg`\n- `0.15.0` — [UPD] support snippet formatting for modern Elixir\n- `0.14.6` — [FIX] persistency flaw when loading [credits @peaceful-james]\n- `0.14.5` — [FIX] `require Logger` in `Hook`\n- `0.14.4` — [FIX] Docs cleanup (credits: @TwistingTwists), `PlantUML` proper entry\n- `0.14.3` — [FIX] Draw diagram in docs\n- `0.14.2` — [FIX] Stop `Events` process\n- `0.14.1` — [FIX] Incorrect detection of superfluous determined transitions\n- `0.14.0` — `Finitomata.ExUnit` improvements\n- `0.13.0` — compile-time helpers for _FSM_, `Finitomata.ExUnit`\n- `0.12.1` — `c:Finitomata.on_start/1` callback\n- `0.11.3` — [FIX] better error message for options (credits @ray-sh)\n- `0.11.2` — [DEBT] exported `Finitomata.fqn/2`\n- `0.11.1` — `Inspect`, `:flowchart`/`:state_diagram` as default parsers, behaviour `Parser`\n- `0.11.0` — `{:ok, state_payload}` return from `on_timer/2`, `:persistent_term` to cache state\n- `0.10.0` — support for several supervision trees with `id`s, experimental support for persistence scaffold\n- `0.9.0` — [FIX] malformed callbacks had the FSM broken\n- `0.8.2` — last error is now kept in the state (credits to @egidijusz)\n- `0.8.1` — improvements to `:finitomata` compiler\n- `0.8.0` — `:finitomata` compiler to warn/hint about not implemented ambiguous transitions\n- `0.7.2` — [FIX] `banged!` transitions must not be determined\n- `0.6.3` — `soft?` events which do not call `on_failure/2` and do not log errors\n- `0.6.2` — `ensure_entry:` option to retry a transition\n- `0.6.1` — code cleanup + `auto_terminate:` option to make `:__end__` transition imminent\n- `0.6.0` — `on_timer/2` and banged imminent transitions\n- `0.5.2` — `state()` type on generated FSMs\n- `0.5.1` — fixed specs [credits @egidijusz]\n- `0.5.0` — all callbacks but `on_transition/4` are optional, accept `impl_for:` param to `use Finitomata`\n- `0.4.0` — allow anonymous FSM instances\n- `0.3.0` — `en_entry/2` and `on_exit/2` optional callbacks\n- `0.2.0` — [Mermaid](https://mermaid.live) support\n\n[Documentation](https://hexdocs.pm/finitomata).","ref":"readme.html#changelog"},{"type":"extras","title":"Finite Automata","doc":"# Finite Automata","ref":"fsm.html"},{"type":"extras","title":"Above and Beyound - Finite Automata","doc":"> If you have issues fully understanding Finite Automata aka _FSM_, here is a one-sentence explanation that might clarify everything.\n>\n> _FSM_ consists of states, transitions, and events, where the state is an adjective (or noun,) the event is a verb and the transition is an adverbial participle.\n\nThis definition, while being mathematically lax, shallow, cursory, and maybe even perfunctory, reveals the whole and allows to grasp the very core thing about _FSM_ nature: it describes a life in the same way the human language desribes it.","ref":"fsm.html#above-and-beyound"},{"type":"extras","title":"Yet Another Library - Finite Automata","doc":"I’m a big fan of [Finite Automata](https://en.wikipedia.org/wiki/Finite-state_machine). Always have been. Whenever we deal with a long-lived objects, they eventually have states and _FSM_ does an enormously great job by attesting consistency, eliminating human errors and leaving the implementation with a business logic only. In a mediocre project, fifty `if`-`then`-`else` conditionals might perform as well as one _FSM_, but unless you are paid for the number of LoCs, _FSM_ is drastically easier to carry on.","ref":"fsm.html#yet-another-library"},{"type":"extras","title":"Internals - Finite Automata","doc":"This library leverages the power of _callbacks_ to not only completely cover the _FSM_ implementation, but also provide a compile-time proof the _FSM_ is valid and functional. One of the most important things this library provides is the _FSM_ description itself, that is fault-tolerant, not error-prone, and easy to grasp. The _FSM_ definition, which is currently supported in both [PlantUML](https://plantuml.com/en/state-diagram) and [Mermaid](https://mermaid.live) syntaxes, would be drawn in the generated docs of the project using this library.\n\nThe consumer of this library initiates a transition by calling somewhat like `transition(object, event)`, then the `GenServer` does its magic and the callback `on_transition/4` gets called. From inside this callback, the consumer implements the business logic and returns the result (the next state to move the _FSM_ to.) There are also syntactic sugar callbacks `on_enter/2`, `on_exit/2`, `on_failure/3`, and `on_terminate/1` to allow easy state change reactions, handling of errors, and a final cleanup respectively.\n\nAll the callbacks do have a default implementation, which would perfectly handle transitions having a single to state and not requiring any additional business logic attached. When needed, this might be turned off.\n\nUpon start, _FSM_ moves to its _initial state_ and sits there awaiting for the transition request. Upon this request, it’d call `on_transition/4` callback and either move to the next state, or remain in the current one, according to the response from the callback. Upon reaching a final state, it would terminate itself, that’s where `on_terminate/1` callback is called from. The process also keeps all the history of states it went through, and might have a payload in its state.","ref":"fsm.html#internals"},{"type":"extras","title":"Cool stuff - Finite Automata","doc":"This library has a compilation-time guarantee the _FSM_ is valid, e. g. has the only one begin state, has at least one end state, all states can be reached, _and_ all the necessary callbacks are defined. That said, if we an _FSM_ has an event initiating transitions from the same state to two different states, and there is no `on_transition/4` clause covering that case, the compile-time error would be raised. On the other hand, if the transition is predetermined and might lead to the only one state, the callback implementation is not mandatory, because there is no trolley problem between these two states.\n\nThe _FSM_ definition allows event names, terminated with bangs and/or question marks. If the event name is terminated with a bang (`init!`,) **and this event is the only one possible from this state,** the event will be called automatically once _FSM_ enters this state. This is handy for moving through initialization or through states which do not require a consumer intervention and might be done immediately after _FSM_ reaches the respective state. If the transition failed in any way (the state has not been left either due to `{:error, any()}` response received from `on_transition/4` _or_ due to other unexpected issue, like if `on_transition/4` raised,) the `on_failure/3` callback would be called and the warning would be printed to the log. To suppress this behaviour and to allow a transition silently fail, the event should have ended with a question mark  (`try_call?`.) The event cannot have both a bang and a question mark in its name.","ref":"fsm.html#cool-stuff"},{"type":"extras","title":"Wiki Example - Finite Automata","doc":"Wikipedia provides a [turnstile as an example](https://commons.wikimedia.org/wiki/File:Turnstile_state_machine_colored.svg#/media/File:Turnstile_state_machine_colored.svg) of _FSM_.\n\n![Turnile State Machine](https://upload.wikimedia.org/wikipedia/commons/9/9e/Turnstile_state_machine_colored.svg)\n\nBelow is the `Finitomata` take on this _FSM_.\n\n```elixir\ndefmodule Turnstile do\n  @fsm \"\"\"\n  built --> |on!| locked\n  locked --> |push| locked\n  locked --> |coin?| unlocked\n  unlocked --> |push| locked\n  unlocked --> |coin?| unlocked\n  unlocked --> |off| destroyed\n  \"\"\"\n\n  use Finitomata, @fsm, auto_terminate: true\n\n  def on_transition(state, :push, _event_payload, state_payload) do\n    if state == :locked, do: electrocute!()\n    {:ok, :locked, state_payload}\n  end\n\n  def on_transition(:locked, :coin?, _event_payload, state_payload) do\n    {:ok, :unlocked, state_payload}\n  end\n\n  def on_transition(:unlocked, :coin, _event_payload, state_payload) do\n    Logger.info(\"Thanks, this coin will be donated to the animal shelter!\")\n    {:error, :unexpected_coin}\n  end\n\n  def on_transition(_, :off, _, state_payload),\n    do: {:ok, :destroyed, state_payload}\n\n  # def on_failure(…), do: …\n\n  # def on_terminate(…), do: …\nend\n```\n\nThe docs for this module would have the following diagram contained (see the `ex_doc` for how to enable _Mermaid_.)\n\n```mermaid\ngraph TD \n  built --> |on!| locked\n  locked --> |push| locked\n  locked --> |coin?| unlocked\n  unlocked --> |push| locked\n  unlocked --> |coin?| unlocked\n  unlocked --> |off| destroyed\n```","ref":"fsm.html#wiki-example"},{"type":"extras","title":"Finitomata Compiler","doc":"# Finitomata Compiler\n\n_Elixir_ provides an ability to introduce a custom [compiler](https://hexdocs.pm/mix/Mix.Task.Compiler.html) for any `mix` project. `Finitomata` leverages this feature, providing a set of [diagnostics](https://hexdocs.pm/mix/Mix.Task.Compiler.Diagnostic.html) for modules declaring the _FSM_.","ref":"compiler.html"},{"type":"extras","title":"Overview - Finitomata Compiler","doc":"Consider the following _FSM_.\n\n```mermaid\ngraph TD \n  idle --> |to_s1!| s1\n  s1 --> |to_s2| s2\n  s1 --> |to_s3| s3\n  s2 --> |to_s1| s3\n  s2 --> |ambiguous| s3\n  s2 --> |ambiguous| s4\n  s3 --> |determined| s3\n  s3 --> |determined| s4\n  s4 --> |determined| s4\n  s4 --> |determined| s5\n```\n\nHere we have transitions which do not require a handler (events `to_s1!`, `to_s1`, `to_s2`, and `to_s3` define the target state, so just sending the event to the _FSM_ in the respective state would be enough.) Also we have `ambiguous` and `determined` events (don’t ask about the name of the latter,) which wouldn’t succeed without the respective `c:Finitomata.on_transition/4` handler declared. This handler must route the event to one of two possible states, conditionally returning `{:ok, :s3, state}` or `{:ok, :s4, state}` based on the state, the payload, or the event payload.\n\nAn attempt to call `ambiguous` event from state `s2` without this handler defined would result in runtime error of the shape\n\n```elixir\n{:error, {:ambiguous_transition, {:s2, :ambiguous}, [:s3. :s4]}}\n```\n\nbut we can do better. Here is the custom compiler shining. Let’s see how the editor would have the code highlighted.\n\n![Editor window with the module using Finitomata](assets/compiler-1.png)","ref":"compiler.html#overview"},{"type":"extras","title":"Warnings, Infos, Hints - Finitomata Compiler","doc":"There are three different types of diagnostics provided by `:finitomata` compiler.\n\n#","ref":"compiler.html#warnings-infos-hints"},{"type":"extras","title":"Warning - Finitomata Compiler","doc":"Warning is reported both to the language server _and_ to console during project compilation. It says “this module declares some ambiguous transitions not handled in the code.” Here is how it looks in the editor.\n\n![Finitomata Warning](assets/compiler-3.png)\n\n#","ref":"compiler.html#warning"},{"type":"extras","title":"Info - Finitomata Compiler","doc":"Info is reported in cases when the compiler is not able to determine whether the ambiguous transition has been covered or not.\n\n![Finitomata Warning](assets/compiler-2.png)\n\n#","ref":"compiler.html#info"},{"type":"extras","title":"Hint - Finitomata Compiler","doc":"Hint is emitted when the compiler is positive about the ambiguous transition has been covered, just for the sake of better user experience.\n\n![Finitomata Warning](assets/compiler-4.png)","ref":"compiler.html#hint"},{"type":"extras","title":"WIP - Finitomata Compiler","doc":"This compiler is very much _WIP_, but it’s already fully functional, and, despite it is not able yet to report complicated cases when the transition seems to be handled, but one or more branches are unreachable, it could be a very handy tool for the additional self-check on whether the _FSM_ is covered or the application might suffer the _ambiguous_, or _not reachable_, or _not allowed_ runtime errors.","ref":"compiler.html#wip"}],"content_type":"text/markdown"}